//------------------------------------------------------------------------------
//
// File:    apsclock
//
// Purpose: Clock for arduino and Nokia 6610
//   
// (c) 2008 Plekhanov Andrey
//
// Initial version 1.0 21.11.2008
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//------------------------------------------------------------------------------

#include <avr/interrupt.h>

#include "Nokia_lcd.h"
#include "DateTime.h"
#include "DateTimeStrings.h"

const unsigned int  CLOCK_BACKGROUND      = WHITE;
const unsigned int  CLOCK_FOREGROUND      = BLACK;
const unsigned char CENTER_X              = 64;
const unsigned char CENTER_Y              = 64;
const float pi = 3.14159265;
const unsigned char SECOND_ARROW_LENGTH_X = 48;
const unsigned char SECOND_ARROW_LENGTH_Y = 48;
const unsigned char MINUTE_ARROW_LENGTH_X = 46;
const unsigned char MINUTE_ARROW_LENGTH_Y = 46;
const unsigned char MINUTE_ARROW_DEEP     = 10;
const unsigned char HOUR_ARROW_LENGTH     = 28;
const unsigned char HOUR_ARROW_DEEP       = 5;
const unsigned char DEBOUNCE_ON           = 5;

// joystick number
const char NO_KEY     = -1;
const char UP_KEY     = 3;
const char LEFT_KEY   = 0;
const char CENTER_KEY = 1;
const char DOWN_KEY   = 2;
const char RIGHT_KEY  = 4;

const unsigned char KEY_EVENT  = 1;
const unsigned char TIME_EVENT = 2;

typedef struct
{
  unsigned char type;
  unsigned char value;
} Event;

#define NUM_KEYS 5

int adc_key_val[ NUM_KEYS ] = { 30, 128, 260, 374, 515 };

volatile char CurrentKey      = -1;
volatile unsigned char bcount = 0;
volatile unsigned char ticks  = 0;

const char MODES_NUMBER = 3;
const char NORMAL_MODE  = 0;
const char DAY_MODE     = 1;
const char MONTH_MODE   = 2;

char mode = NORMAL_MODE;

const char MENU_ITEMS = 8;

unsigned long prevtime = 0;
char           prevDay = -1;

const int TIMER1_F = 5000;

Nokia_lcd lcd = Nokia_lcd();
DateTimeClass DateTime = DateTimeClass();
DateTimeStringsClass DateTimeStrings = DateTimeStringsClass();

void GetEvent( Event & event );

void DrawClock( const bool update )
{
  if( update ) {
    lcd.cLCD_Box( 0, 0, 131, 131, FILL, CLOCK_BACKGROUND );

    lcd.cLCD_String("11", 20, 2, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("12", 58, 2, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("1", 100, 2, CLOCK_FOREGROUND, CLOCK_BACKGROUND );

    unsigned char y1 = 24;
    unsigned char y2 = 60;
    unsigned char y3 = 96;
    unsigned char y4 = 122;

    if( mode != NORMAL_MODE ) {
      y1 = y1 - 2;
      y2 = y2 - 4;
      y3 = y3 - 6;
      y4 = y4 - 8;
    }

    lcd.cLCD_String("2", 120, y1, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("3", 120, y2, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("4", 120, y3, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("5", 100, y4, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("6",  60, y4, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("7",  22, y4, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("8",   4, y3, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("9",   4, y2, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
    lcd.cLCD_String("10",  0, y1, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  if( DateTime.available() ) {
    if( ( prevtime != DateTime.now() ) || update ) {

      unsigned char y1 = 120;
      unsigned char second_arrow_length_x = SECOND_ARROW_LENGTH_X;
      unsigned char second_arrow_length_y = SECOND_ARROW_LENGTH_Y;
      unsigned char minute_arrow_length_x = MINUTE_ARROW_LENGTH_X;
      unsigned char minute_arrow_length_y = MINUTE_ARROW_LENGTH_Y;
      unsigned char center_x = CENTER_X;
      unsigned char center_y = CENTER_Y;

      if( mode != NORMAL_MODE ) {
        y1 = y1 - 8;
        center_y = center_y - 4;
        second_arrow_length_y = second_arrow_length_y - 4;
        minute_arrow_length_y = minute_arrow_length_y - 4;
      }

      lcd.cLCD_Box( 16, 16, 120, y1, FILL, CLOCK_BACKGROUND );

      unsigned char x_end;
      unsigned char y_end;
      unsigned char x_start;
      unsigned char y_start;
    
      float CosX = cos( DateTime.Second * 2.0 * pi / 60.0 - pi / 2.0 );
      float SinX = sin( DateTime.Second * 2.0 * pi / 60.0 - pi / 2.0 );

      x_end = center_x + static_cast<unsigned char>( second_arrow_length_x * CosX );

      y_end = center_y + static_cast<unsigned char>( second_arrow_length_y * SinX );

      lcd.cLCD_Line( center_x, center_y, x_end, y_end, CLOCK_FOREGROUND );

      CosX = cos( ( DateTime.Minute + static_cast<float>( DateTime.Second ) / 60.0 ) * 2.0 * pi / 60.0 - pi / 2.0 );

      SinX = sin( ( DateTime.Minute + static_cast<float>( DateTime.Second ) / 60.0 ) * 2.0 * pi / 60.0 - pi / 2.0 );

      x_end = center_x + static_cast<unsigned char>( minute_arrow_length_x * CosX );

      y_end = center_y + static_cast<unsigned char>( minute_arrow_length_y * SinX );

      x_start = center_x - static_cast<unsigned char>( MINUTE_ARROW_DEEP * CosX );

      y_start = center_y - static_cast<unsigned char>( MINUTE_ARROW_DEEP * SinX );

      lcd.cLCD_Line( x_start, y_start, x_end, y_end, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start + 1, y_start, x_end + 1, y_end, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start - 1, y_start, x_end - 1, y_end, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start, y_start + 1, x_end, y_end + 1, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start, y_start - 1, x_end, y_end - 1, CLOCK_FOREGROUND );

      CosX = cos( ( DateTime.Hour % 12 + static_cast<float>( DateTime.Minute ) / 60.0 +
                    static_cast<float>( DateTime.Second ) / 3600.0 ) * 2.0 * pi / 12.0 - pi / 2.0 );

      SinX = sin( ( DateTime.Hour % 12 + static_cast<float>( DateTime.Minute ) / 60.0 +
                    static_cast<float>( DateTime.Second ) / 3600.0 ) * 2.0 * pi / 12.0 - pi / 2.0 );

      x_end = center_x + static_cast<unsigned char>( HOUR_ARROW_LENGTH * CosX );

      y_end = center_y + static_cast<unsigned char>( HOUR_ARROW_LENGTH * SinX );

      x_start = center_x - static_cast<unsigned char>( HOUR_ARROW_DEEP * CosX );

      y_start = center_y - static_cast<unsigned char>( HOUR_ARROW_DEEP * SinX );

      lcd.cLCD_Line( x_start, y_start, x_end, y_end, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start + 1, y_start, x_end + 1, y_end, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start - 1, y_start, x_end - 1, y_end, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start, y_start + 1, x_end, y_end + 1, CLOCK_FOREGROUND );
      lcd.cLCD_Line( x_start, y_start - 1, x_end, y_end - 1, CLOCK_FOREGROUND );

      if( ( prevDay != DateTime.Day ) || update ) {
        if( ( mode == DAY_MODE ) || ( mode == MONTH_MODE ) ) {
          char Str[ 32 ];

          itoa( DateTime.Day, Str, 10 );        

          strcat( Str, " " );

          if( mode == DAY_MODE ) {        
            strcat( Str, DateTimeStrings.dayStr( DateTime.DayofWeek ) );
          }
          else {
            if( mode == MONTH_MODE ) {        
              strcat( Str, DateTimeStrings.monthStr( DateTime.Month ) );
            }
          }

          char len = strlen( Str );

          lcd.cLCD_String( Str, 64 - 4 * len, 122, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
        }
        
        prevDay = DateTime.Day;
      }

      prevtime = DateTime.now();
    }
  }
}

void DrawMenu( const char menu_item, const char sec, const char min,
               const char hour, const char day,
               const char month, const int year )
{
  char Str[ 32 ];

  lcd.cLCD_Box( 60, 20, 131, 80, FILL, CLOCK_BACKGROUND );

  itoa( sec, Str, 10 );

  if( menu_item == 0 ) {
    lcd.cLCD_String( Str, 60, 20, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( Str, 60, 20, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  itoa( min, Str, 10 );

  if( menu_item == 1 ) {
    lcd.cLCD_String( Str, 60, 30, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( Str, 60, 30, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  itoa( hour, Str, 10 );

  if( menu_item == 2 ) {
    lcd.cLCD_String( Str, 60, 40, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( Str, 60, 40, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  itoa( day, Str, 10 );

  if( menu_item == 3 ) {
    lcd.cLCD_String( Str, 60, 50, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( Str, 60, 50, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  if( menu_item == 4 ) {
    lcd.cLCD_String( DateTimeStrings.monthStr( month ),
                     60, 60, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( DateTimeStrings.monthStr( month ),
                     60, 60, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  itoa( year, Str, 10 );

  if( menu_item == 5 ) {
    lcd.cLCD_String( Str, 60, 70, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( Str, 60, 70, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  lcd.cLCD_Box( 40, 90, 90, 110, FILL, CLOCK_BACKGROUND );

  if( menu_item == 6 ) {
    lcd.cLCD_String( "OK", 55, 90, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( "OK", 55, 90, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }

  if( menu_item == 7 ) {
    lcd.cLCD_String( "Cancel", 40, 100, CLOCK_BACKGROUND, CLOCK_FOREGROUND );
  }
  else {
    lcd.cLCD_String( "Cancel", 40, 100, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  }
}

void SetupClock( void )
{
  char sec   = DateTime.Second;
  char min   = DateTime.Minute;
  char hour  = DateTime.Hour;
  char day   = DateTime.Day;
  char month = DateTime.Month;
  int  year  = DateTime.Year + 1900;

  lcd.cLCD_Box( 0, 0, 131, 131, FILL, CLOCK_BACKGROUND );

  lcd.cLCD_String("sec",   10, 20, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  lcd.cLCD_String("min",   10, 30, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  lcd.cLCD_String("hour",  10, 40, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  lcd.cLCD_String("day",   10, 50, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  lcd.cLCD_String("month", 10, 60, CLOCK_FOREGROUND, CLOCK_BACKGROUND );
  lcd.cLCD_String("year",  10, 70, CLOCK_FOREGROUND, CLOCK_BACKGROUND );

  char menu_item = 0;

  while( 1 ) {
    DrawMenu( menu_item, sec, min, hour, day, month, year );

    unsigned char key = GetKey();

    switch( key ) {
      case DOWN_KEY:
        menu_item++;

        if( menu_item >= MENU_ITEMS ) {
          menu_item = 0;
        }
      break;
      case UP_KEY:
        menu_item--;

        if( menu_item < 0 ) {
          menu_item = MENU_ITEMS - 1;
        }
      break;
      case LEFT_KEY:
        switch( menu_item ) {
          case 0:
            sec--;

            if( sec < 0 ) {
              sec = 59;
            }
          break;
          case 1:
            min--;

            if( min < 0 ) {
              min = 59;
            }
          break;
          case 2:
            hour--;

            if( hour < 0 ) {
              hour = 23;
            }
          break;
          case 3:
            day--;

            if( day < 1 ) {
              day = 31;
            }
          break;
          case 4:
            month--;

            if( month < 0 ) {
              month = 11;
            }
          break;
          case 5:
            year--;

            if( year < 2008 ) {
              year = 2020;
            }
          break;
          default:;
        }
      break;
      case RIGHT_KEY:
        switch( menu_item ) {
          case 0:
            sec++;

            if( sec >= 60 ) {
              sec = 0;
            }
          break;
          case 1:
            min++;

            if( min >= 60 ) {
              min = 0;
            }
          break;
          case 2:
            hour++;

            if( hour >= 24 ) {
              hour = 0;
            }
          break;
          case 3:
            day++;

            if( day >= 32 ) {
              day = 1;
            }
          break;
          case 4:
            month++;

            if( month >= 12 ) {
              month = 0;
            }
          break;
          case 5:
            year++;

            if( year >= 2021 ) {
              year = 2008;
            }
          break;
          default:;
        }
      break;
      case CENTER_KEY:
        if( menu_item == 6 ) {
          time_t t = DateTime.makeTime( sec, min, hour, day, month, year );
          DateTime.sync( t );  
          return;
        }

        if( menu_item == 7 ) {
          return;
        }
      break;
      default:;
    }
  }
}

void setup( void )
{
  DDRB=0x2F;

  // Setup timer1 -- Prescaler/64
  TCCR1A = 0;
  TCCR1B = _BV(WGM12) | _BV(CS11) | _BV(CS10);
  OCR1A  = TIMER1_F;
  TCNT1  = 0;
  TIMSK1 = _BV(OCIE1A);
  
  SREG |= 1 << SREG_I;

  LCD_BACKLIGHT( 1 );
  lcd.cLCD_Init();

  DrawClock( true );

  time_t t = DateTime.makeTime( 0, 0, 0, 22, 10, 2008 );
  DateTime.sync( t );
}

void loop( void )
{
  Event event;

  GetEvent( event );

  switch( event.type ) {
    case KEY_EVENT:
      switch( event.value ) {
        case UP_KEY:
          mode++;

          if( mode >= MODES_NUMBER ) {
            mode = 0;
          }

          DrawClock( true );
        break;
        case DOWN_KEY:
          mode--;

          if( mode < 0 ) {
            mode = MODES_NUMBER - 1;
          }

          DrawClock( true );
        break;
        case LEFT_KEY:
        break;
        case RIGHT_KEY:
        break;
        case CENTER_KEY:
          SetupClock();
          DrawClock( true );
        break;
        default:;
      }
    break;
    case TIME_EVENT:
      DrawClock( false );
    break;
    default:;
  }
}

char get_key( unsigned int input )
{
  char k;
    
  for( k = 0; k < NUM_KEYS; k++ ) {
    if( input < adc_key_val[ k ] ) {
      return( k );
    }
  }
  
  return( -1 );
}

void GetEvent( Event & event )
{
  while( 1 ) {
    if( ticks ) {
      event.type = TIME_EVENT;
      event.value = ticks;

      ticks = 0;

      break;
    }

    if( CurrentKey != NO_KEY ) {
      event.type = KEY_EVENT;
      event.value = CurrentKey;
  
      CurrentKey = NO_KEY;

      break;
    }
  }
}

unsigned char GetKey( void )
{
  Event event;

  while( 1 ) {
    GetEvent( event );

    if( event.type == KEY_EVENT ) {
      return( event.value );
    }
  }
}

// Timer1 interrupt routine -
// (64*5000)/160000000 = 20ms interval

ISR(TIMER1_COMPA_vect)
{
  int adc_key_in;
  char key_in;
  byte i;

  ticks++;
  
  adc_key_in = analogRead( 0 );
  
  key_in = get_key( adc_key_in );
  
  if( key_in >= 0 ) {
    if( bcount < DEBOUNCE_ON ) {
      bcount++;
    }
    else {
      if( CurrentKey == NO_KEY ) {
        CurrentKey = key_in;
        bcount = 0;        
      }
    }
  }
  else {
    if( bcount > 0 ) {
      bcount--; 
    }
  }
}
